{*********************************************************}
{*                    TPDATE.PAS 5.05                    *}
{*********************************************************}

unit TpDate;
  {-Based in part on the public domain units DATES.PAS by Scott Bussinger and
    JULIAN.PAS by Carley Phillips. Many thanks to both.}

interface


type
  TDate = LongInt;
  Time = LongInt;

const
    MinYear = 1600;
    MaxYear = 3999;
    MinDate = $00000000;     {= 01/01/1600}
    MaxDate = $000D6025;     {= 12/31/3999}
    Date1900 = $0001AC05;    {= 01/01/1900}
    Date1980 = $00021E28;    {= 01/01/1980}
    BadDate = $FFFFFFFF;

  MinTime = 0;               {= 00:00:00 am}
  MaxTime = 86399;           {= 23:59:59 pm}
  BadTime = $FFFFFFFF;

  SecondsInDay = 86400;      {number of seconds in a day}
  SecondsInHour = 3600;      {number of seconds in an hour}
  SecondsInMinute = 60;      {number of seconds in a minute}
  HoursInDay = 24;           {number of hours in a day}
  MinutesInHour = 60;        {number of minutes in an hour}

var
  DefaultYear : Integer;     {default year--used by DateStringToDMY}




  {-------julian date routines---------------}

function ValidDate(Day, Month, Year : Integer) : Boolean;
  {-Verify that day, month, year is a valid date}

function DaysInMonth(Month, Year : Integer) : Integer;
  {-Return the number of days in the specified month of a given year}

function DMYtoDate(Day, Month, Year : Integer) : TDate;
  {-Convert from day, month, year to a julian date}

procedure DateToDMY(Julian : TDate; var Day, Month, Year : Integer);
  {-Convert from a julian date to day, month, year}

function IncDate(Julian : TDate; Days, Months, Years : Integer) : TDate;
  {-Add (or subtract) the number of days, months, and years to a date}

procedure DateDiff(Date1, Date2 : TDate; var Days, Months, Years : Integer);
  {-Return the difference in days,months,years between two valid julian dates}

{ ---------------- String convertion utilites ------------------}

function Long2Str( V : longint) : string;
  {- Converts longint to string}


  {==========================================================================}

implementation

const
    First2Months = 59;         {1600 was a leap year}

  function IsLeapYear(Year : Integer) : Boolean;
    {-Return True if Year is a leap year}
  begin
    IsLeapYear := (Year mod 4 = 0) and (Year mod 4000 <> 0) and
      ((Year mod 100 <> 0) or (Year mod 400 = 0));
  end;

  function DaysInMonth(Month, Year : Integer) : Integer;
    {-Return the number of days in the specified month of a given year}
  begin
    if Word(Year) < 100 then
      Inc(Year, 1900);
    case Month of
      1, 3, 5, 7, 8, 10, 12 :
        DaysInMonth := 31;
      4, 6, 9, 11 :
        DaysInMonth := 30;
      2 :
        DaysInMonth := 28+Ord(IsLeapYear(Year));
    else
      DaysInMonth := 0;
    end;
  end;

  function ValidDate(Day, Month, Year : Integer) : Boolean;
    {-Verify that day, month, year is a valid date}
  begin
    if Word(Year) < 100 then
      Inc(Year, 1900);

    if (Day < 1) or (Year < MinYear) or (Year > MaxYear) then
      ValidDate := False
    else case Month of
      1..12 :
        ValidDate := Day <= DaysInMonth(Month, Year);
    else
      ValidDate := False;
    end
  end;

  function DMYtoDate(Day, Month, Year : Integer) : TDate;
    {-Convert from day, month, year to a julian date}
  begin
    if Word(Year) < 100 then
      Inc(Year, 1900);

    if not ValidDate(Day, Month, Year) then
      DMYtoDate := BadDate
    else if (Year = MinYear) and (Month < 3) then
      if Month = 1 then
        DMYtoDate := Pred(Day)
      else
        DMYtoDate := Day+30
    else begin
      if Month > 2 then
        Dec(Month, 3)
      else begin
        Inc(Month, 9);
        Dec(Year);
      end;
      Dec(Year, MinYear);
      DMYtoDate :=
        ((LongInt(Year div 100)*146097) div 4)+
        ((LongInt(Year mod 100)*1461) div 4)+
        (((153*Month)+2) div 5)+Day+First2Months;
    end;
  end;

  procedure DateToDMY(Julian : TDate; var Day, Month, Year : Integer);
    {-Convert from a julian date to month, day, year}
  var
    I, J : LongInt;
  begin
    if Julian = BadDate then begin
      Day := 0;
      Month := 0;
      Year := 0;
    end
    else if Julian <= First2Months then begin
      Year := MinYear;
      if Julian <= 30 then begin
        Month := 1;
        Day := Succ(Julian);
      end
      else begin
        Month := 2;
        Day := Julian-30;
      end;
    end
    else begin
      I := (4*LongInt(Julian-First2Months))-1;
      J := (4*((I mod 146097) div 4))+3;
      Year := (100*(I div 146097))+(J div 1461);
      I := (5*(((J mod 1461)+4) div 4))-3;
      Month := I div 153;
      Day := ((I mod 153)+5) div 5;
      if Month < 10 then
        Inc(Month, 3)
      else begin
        Dec(Month, 9);
        Inc(Year);
      end;
      Inc(Year, MinYear);
    end;
  end;

  function IncDate(Julian : TDate; Days, Months, Years : Integer) : TDate;
    {-Add (or subtract) the number of months, days, and years to a date.
      Months and years are added before days. No overflow/underflow
      checks are made}
  var
    Day, Month, Year, Day28Delta : Integer;
  begin
    DateToDMY(Julian, Day, Month, Year);
    Day28Delta := Day-28;
    if Day28Delta < 0 then
      Day28Delta := 0
    else
      Day := 28;
    Inc(Month, Months-1);
    Inc(Year, Years+(Month div 12)-Ord(Month < 0));
    Month := Succ(Month mod 12);
    if Month <= 0 then
      Inc(Month, 12);
    Julian := DMYtoDate(Day, Month, Year);
    if Julian <> BadDate then begin
      Inc(Julian, Days);
      Inc(Julian, Day28Delta);
    end;
    IncDate := Julian;
  end;

  procedure DateDiff(Date1, Date2 : TDate; var Days, Months, Years : Integer);
    {-Return the difference in days,months,years between two valid julian dates}
  var
    Day1, Day2, Month1, Month2, Year1, Year2 : Integer;
    TmpDate : TDate;
  begin
    {we want Date2 > Date1}
    if Date1 > Date2 then begin
      TmpDate := Date1;
      Date1 := Date2;
      Date2 := TmpDate;
      end;

    {convert dates to day,month,year}
    DateToDMY(Date1, Day1, Month1, Year1);
    DateToDMY(Date2, Day2, Month2, Year2);

    {days first}
    if Day2 < Day1 then begin
      Dec(Month2);
      if Month2 = 0 then begin
        Month2 := 12;
        Dec(Year2);
      end;
      Inc(Day2, DaysInMonth(Month2, Year2));
    end;
    Days := Day2-Day1;

    {now months and years}
    if Month2 < Month1 then begin
      Inc(Month2, 12);
      Dec(Year2);
    end;
    Months := Month2-Month1;
    Years := Year2-Year1;
  end;

function Long2Str( V : longint) : string;
var S : string;
begin
  str( V, S);
  Long2Str := S;
end;

end.
